@@ -102,6 +102,8 @@ zenity_entry (ZenityData *data, ZenityEntryData *entry_data)
 
 	if (n_entries > 1)
 	{
+		G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 		GtkWidget *child;
 
 		entry = gtk_combo_box_text_new_with_entry ();
@@ -123,6 +125,8 @@ zenity_entry (ZenityData *data, ZenityEntryData *entry_data)
 		g_signal_connect (child, "activate",
 			G_CALLBACK (zenity_entry_combo_activate_default),
 			GTK_WINDOW (dialog));
+
+		G_GNUC_END_IGNORE_DEPRECATIONS
 	}
 	else
 	{
@@ -148,11 +152,11 @@ zenity_entry (ZenityData *data, ZenityEntryData *entry_data)
 
 	zenity_util_show_dialog (dialog);
 
-	if (data->timeout_delay > 0) {
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			dialog);
+	if (data->timeout_delay > 0)
+	{
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
+
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
@@ -163,7 +167,9 @@ zenity_entry_dialog_output (void)
 
 	if (n_entries > 1)
 	{
+		G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 		text = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT(entry));
+		G_GNUC_END_IGNORE_DEPRECATIONS
 	}
 	else
 	{
@@ -181,13 +187,13 @@ static void
 zenity_entry_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 {
 	ZenityData *zen_data = data;
-	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
+	int response = zenity_util_parse_dialog_response (rstr);
 
 	switch (response)
 	{
 		case ZENITY_OK:
 			zenity_entry_dialog_output ();
-			zenity_util_exit_code_with_data (ZENITY_OK, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			break;
 
 		case ZENITY_CANCEL:
@@ -204,10 +210,7 @@ zenity_entry_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			break;
 
 		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-				printf ("%s\n", zen_data->extra_label[response]);
-			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
