@@ -31,32 +31,6 @@
 
 static void zenity_msg_dialog_response (GtkWidget *widget, char *rstr, gpointer data);
 
-/* FIXME - Is this still necessary with gtk4? */
-static void
-zenity_label_widget_clipboard_selection (GtkWidget *widget)
-{
-	/* Workaround hotfix for suspected toolkit issue:
-	   since focus change of the dialog's focussed widget (text)
-	   somehow currently chooses to destroy
-	   a pre-existing (read: foreign, user-initiated) X11 primary selection
-	   (via gtk_label_select_region() -> ...
-	   -> gtk_clipboard_set_contents()/gtk_clipboard_clear()),
-	   we need to ensure
-	   that the widget does have its gtk-label-select-on-focus property off,
-	   in order to avoid having the label become selected automatically
-	   and thereby having pre-existing clipboard content nullified.
-	   Side note: this selection issue only applies to widgets
-	   which have both
-					<property name="can_focus">True</property>
-					<property name="selectable">True</property>
-	   .
-	 */
-	g_object_set (gtk_widget_get_settings (widget),
-		"gtk-label-select-on-focus",
-		FALSE,
-		NULL);
-}
-
 void
 zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
 {
@@ -200,7 +174,6 @@ zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
 			gtk_label_set_markup (GTK_LABEL (text),
 					g_strcompress (msg_data->dialog_text));
 		}
-		zenity_label_widget_clipboard_selection (GTK_WIDGET (text));
 	}
 
 	if (msg_data->ellipsize)
@@ -219,12 +192,18 @@ zenity_msg (ZenityData *data, ZenityMsgData *msg_data)
 
 	zenity_util_show_dialog (dialog);
 
-	if (data->timeout_delay > 0) {
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			NULL);
+	if (data->timeout_delay > 0)
+	{
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
 
+	/* Disable select-on-focus for labels to avoid primary selection getting
+	 * overwritten inadvertently, which can annoy some users.
+	 */
+	g_object_set (gtk_settings_get_default (),
+		"gtk-label-select-on-focus", FALSE,
+		NULL);
+
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
@@ -232,12 +211,12 @@ static void
 zenity_msg_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 {
 	ZenityData *zen_data = data;
-	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
+	int response = zenity_util_parse_dialog_response (rstr);
 
 	switch (response)
 	{	
 		case ZENITY_OK:
-			zenity_util_exit_code_with_data (ZENITY_OK, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			break;
 
 		case ZENITY_CANCEL:
@@ -248,15 +227,12 @@ zenity_msg_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
 			break;
 
-		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-				printf ("%s\n", zen_data->extra_label[response]);
+		case ZENITY_TIMEOUT:
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_TIMEOUT);
+			break;
 
-			/* This is an odd exit code for extra labels, but it is maintained
-			 * for backwards compatibility with zenity <= 3.x.
-			 */
-			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+		default:
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
